module HC_SR04_core(clk, rst, en, echo, dis_time_us,trig); // 检测回波高电平持续时间
input wire clk, rst, en, echo;
output wire trig;
output wire [15 : 0] dis_time_us;

parameter S0 = 2'b00, S1 = 2'b01, S2 = 2'b10; // 状态定义 S0:闲置, S1:开始测距计数, S2:结束测距计数

logic echo_logic1, echo_logic2,trig_reg;
logic unsigned [1:0] state;
logic unsigned [15:0] count;
logic unsigned [15:0] count_trig;
logic unsigned [7:0] count_1m;
logic unsigned [15:0] dis_logic;

logic clk_1m;

assign trig = trig_reg;

always_ff@(posedge clk, negedge rst)
begin : CLK_1M_DIVIDE
    if(~rst) begin
      count_1m <= 0;
    end else if(count_1m >= 8'd99)begin
		count_1m <= 0;
	end else begin
		count_1m <= count_1m + 1'b1;
	end
end : CLK_1M_DIVIDE


always_ff@(posedge clk, negedge rst)
begin : CLK_1M_GEN
    if(~rst) begin
      clk_1m <= 1'b0;
    end else if(count_1m <= 8'd50-1)begin
		clk_1m <= 1'b0;
	end else begin
		clk_1m <= 1'b1;
	end
end : CLK_1M_GEN

always_ff@(posedge clk_1m, negedge rst)
begin : echo_logic
    if(~rst)
    begin
        echo_logic1 <= 0;
        echo_logic2 <= 0;
    end else begin
		  echo_logic1 <= echo;          // 当前
        echo_logic2 <= echo_logic1;     // 后一个
	end 
end : echo_logic

assign start = echo_logic1 & (~echo_logic2);  //检测posedge
assign finish = (~echo_logic1) & echo_logic2; //检测negedge

always_ff@(posedge clk_1m, negedge rst)
begin : count_trig_gen
    if (~rst) begin
        count_trig <= 0;
		  trig_reg <= 0;
	end else if(en) begin
		if(count_trig >= 16'd50000) begin
			count_trig <= 0;
			trig_reg <= 0;
		end else begin
			count_trig <= count_trig + 1;
			if(count_trig <= 16'd9) begin
				trig_reg <= 1;
			end else begin
				trig_reg <= 0;
			end
		end
	end else begin
		count_trig <= 0;
		trig_reg <= 0;
	end
end  : count_trig_gen


always_ff@(posedge clk_1m, negedge rst)
begin : dis_time_us_gen
    if(~rst)
    begin
        count <= 0;
        dis_logic <= 0;
        state <= S0;
    end else begin
        case(state)
        S0:begin
               if (start) begin // 检测到上升沿
                  state <= S1; //S1
						count <= 0;
               end else begin
                  count <= 0;
						state <= S0;
					end
            end
        S1:begin
               if (finish) begin// 检测到下降沿
                    state <= S2; //S2
					     count <= count;
					end
               else if(count >= 16'd24000) begin 
						count <= count;
						state <= S2; 
					end
					else begin
                  count <= count + 1;
						state <= S1;
                end
            end
        S2:begin
                dis_logic <= count; // 缓存计数结果
                count <= 0;
                state <= S0; //S0
            end
		default: begin
			dis_logic <= 0;
			count <= 0;
			state <= S0;
			end
        endcase
    end
end : dis_time_us_gen
 

assign dis_time_us = dis_logic; // 距离

endmodule : HC_SR04_core
