/*
###############################################################################
# Copyright (c) 2017, PulseRain Technology LLC 
#
# This program is distributed under a dual license: an open source license, 
# and a commercial license. 
# 
# The open source license under which this program is distributed is the 
# GNU Public License version 3 (GPLv3).
#
# And for those who want to use this program in ways that are incompatible
# with the GPLv3, PulseRain Technology LLC offers commercial license instead.
# Please contact PulseRain Technology LLC (www.pulserain.com) for more detail.
#
###############################################################################
*/


//=============================================================================
// Remarks:
//    Wishbone wrapper for PWM
//=============================================================================

`include "config.vh"
`include "common.vh"
//`include "PWM.vh"


`default_nettype none

module wb_HC_SR04 
					  #(parameter
                         REG_ADDR_CSR,
					     REG_ADDR_DATA
                      )
		(

        
        //=======================================================================
        // clock / reset
        //=======================================================================
        
        input   wire                                clk,
        input   wire                                reset_n,

        //=======================================================================
        // Wishbone Interface (FASM synchronous RAM dual port model)
        //=======================================================================
            
        input  wire                                 stb_i,
        input  wire                                 we_i,
		

		
        
        input  wire  unsigned [`MM_REG_ADDR_BITS - 1 : 0]  adr_wr_i,
        input  wire  unsigned [`MM_REG_ADDR_BITS - 1 : 0]  adr_rd_i,
       // input  wire  unsigned [PWM_DATA_LEN - 1 : 0]  dat_i,
	    input  wire  unsigned [7 : 0]                      dat_i,
        output wire  unsigned [15 : 0]                     dat_o,
        output wire                                        ack_o,
    
        //=======================================================================
        // dis_time_us output
        //=======================================================================
		input  wire 								       echo,       
        output wire                                        trig	
        
);


    
    
    //=======================================================================
    // Signals 
    //=======================================================================
    //    logic unsigned [NUM_OF_PWM - 1 : 0][15 : 0]  reg_on;
    //    logic unsigned [NUM_OF_PWM - 1 : 0][15 : 0]  reg_off;
        
    
      
        logic                                        we;
        logic                                        re;
         
        logic  unsigned [7 : 0]                      write_addr;
        logic  unsigned [7 : 0]                      read_addr;
        logic  unsigned [7 : 0]                      dat_i_reg;
        logic  unsigned [15 : 0]  						  dat_o_reg;
        
		  logic  unsigned [15 : 0]  						  read_data_reg;
		  
        logic unsigned [7 : 0]                       reg_csr;
  
		  wire 										           enHC_SR04;   
        
        wire  unsigned [2 : 0]                       sub_addr;
		  wire  unsigned [15 : 0]                      dis_time_us;
            
    //=======================================================================
    // registers and flags
    //=======================================================================
        
        //assign we = stb_i & we_i;
        assign re = stb_i & (~we_i);
        //assign write_addr = adr_wr_i;
        assign read_addr  = adr_rd_i;
        
        always_ff @(posedge clk, negedge reset_n) begin : rw_proc
            if (!reset_n) begin
                we <= 0;
        //      re <= 0;
                write_addr <= 0;
        //      read_addr <= 0;
                dat_i_reg <= 0;
            end else begin
                we <= stb_i & we_i;
        //      re <= stb_i & (~we_i);
                write_addr <= adr_wr_i;
        //      read_addr  <= adr_rd_i;
                dat_i_reg <= dat_i;
    
            end
            
        end : rw_proc
        
        
        always_ff @(posedge clk, negedge reset_n) begin : reg_csr_proc
            if (!reset_n) begin
                reg_csr <= 0;
            end else if (we & (~(|(write_addr ^ REG_ADDR_CSR)))) begin
                reg_csr <= dat_i_reg;  
            end else begin
                reg_csr <= reg_csr;
            end
        end : reg_csr_proc
        
        
		  assign enHC_SR04 = reg_csr[0];
        //assign sub_addr = reg_csr[3 : 1];		   
       
                
          /*      
                always_ff @(posedge clk, negedge reset_n) begin : reg_resolution_proc
                    if (!reset_n) begin
                        dat_o_reg <= 0;
                    end else if ((sub_addr == 3'b001)) begin  //&& (read_addr==REG_ADDR_DATA)
                        dat_o_reg <= dis_time_us; // dis_time_us;
                    end else begin
								dat_o_reg <= 1000;
						  end
                end : reg_resolution_proc
        */
					assign dat_o = dis_time_us;
		
              
                
        HC_SR04_core  HC_SR04_core (
				    .clk(clk),
                .rst(reset_n), 
					.en(enHC_SR04),
					.echo(echo),
               .dis_time_us(dis_time_us),
					.trig(trig)
                );
                
       

endmodule : wb_HC_SR04


`default_nettype wire
